#! /usr/bin/env node

'use strict'

const program = require('commander')
const path = require('path')
const execa = require('execa')
const argv = require('yargs').argv
const ui = require('cliui')()
const Table = require('cli-table3')
const chalk = require('chalk')
const pkg = require('../package.json')
// const checkVersion = require('../lib/checkVersion')
const updateNotifier = require('update-notifier')

// 可用的 abc 命令
const availableCommands = [
  { cmd: 'init', desc: '根据模板生成项目结构' },
  { cmd: 'dll', desc: '第三方模块打包成dll' },
  { cmd: 'daily', desc: '日常代码构建' },
  { cmd: 'dev', desc: '开启本地调试' },
  { cmd: 'build', desc: '线上代码构建' },
  { cmd: 'babel', desc: 'babel转义' },
  { cmd: 'publish', desc: '组件发布' }
]

// 可用的生成器命令
const availableSubCmd = [
  { cmd: 'rc', desc: 'react 组件' },
  { cmd: 'agg-vue', desc: 'agg vue 模板' },
  { cmd: 'agg-h5', desc: 'agg h5 模板' },
  { cmd: 'agg-mp', desc: '微信小程序' },
  { cmd: 'vue-project', desc: 'vue 工程模板' },
  { cmd: 'rc-frame-app', desc: 'react 框架模板' },
  { cmd: 'rc-micro-app', desc: 'react 微应用模板' }
]

/**
 * 获取可用的命令
 * @param {String} cmd
 */
function getCmd(cmd) {
  return availableCommands.find((it) => {
    return it.cmd === cmd
  })
}

/**
 * 获取可用的生成器命令
 * @param {String} cmd
 */
function getSubCmd(cmd) {
  return availableSubCmd.find((it) => {
    return it.cmd === cmd
  })
}

/**
 * 打印可用的生成器命令
 */
function printAvailableGenerators() {
  const table = new Table({
    head: [
      chalk.hex('#ff4c4c').bold('当前可用的生成器'),
      chalk.hex('#ff4c4c').bold('描述')
    ]
  })

  availableSubCmd.forEach((it) => {
    table.push([
      chalk.hex('#2dde98').bold(it.cmd),
      chalk.hex('#2dde98').bold(it.desc)
    ])
  })

  console.log(table.toString())
}

function execCmd(cmd) {
  const filePath = require.resolve(`../server/${cmd}`)
  // 需要透传命令行参数
  // shell.exec(`node ${filePath} ${process.argv.join(' ')}`)
  execa.shell(`node ${filePath} ${process.argv.join(' ')}`, {
    stdio: 'inherit'
  })
}

program
  .version(pkg.version)
  .option('-G, --generators', '显示目前支持的项目模板')
  .option('--config <type>', '指定配置文件')
  .usage('[command] [options]')

program
  .command('*')
  .description('讯盟-前端项目模板生成器和构建器工具')
  .action(function(env) {
    program.help()
    process.exit(0)
  })

program
  .command('init [generator-name]')
  .description(getCmd('init').desc)
  .action((env, options) => {
    const { _ } = argv
    const subCmd = (_[1] === 'rc' ? '' : _[1]) || ''
    const otherParams = _.length > 2 ? _.slice(2) : []

    if (subCmd && !getSubCmd(subCmd)) {
      printAvailableGenerators()

      process.exit(0)
    }

    // 版本更新检测
    const notifier = updateNotifier({ pkg })
    if (notifier.update && notifier.update.latest !== pkg.version) {
      const color = require('ansicolors')
      const useColor = true
      const useUnicode = true
      const old = notifier.update.current
      const latest = notifier.update.latest
      let type = notifier.update.type
      if (useColor) {
        switch (type) {
          case 'major':
            type = color.red(type)
            break
          case 'minor':
            type = color.yellow(type)
            break
          case 'patch':
            type = color.green(type)
            break
        }
      }
      notifier.notify({
        message:
          `New ${type} version of ${pkg.name} available! ${
            useColor ? color.red(old) : old
          } ${useUnicode ? '→' : '->'} ${
            useColor ? color.green(latest) : latest
          }\n` +
          `Run ${
            useColor
              ? color.green(`npm install -g ${pkg.name}`)
              : `cnpm i -g ${pkg.name}`
          } to update!`
      })
    }

    try {
      execa.shell(
        `yo --no-insight abc${
          subCmd ? ':' + subCmd : ''
        } ${otherParams} --cwd=${process.cwd()}`,
        { stdio: 'inherit', cwd: __dirname }
      )
    } catch (ex) {
      // TODO
    }
  })

program
  .command('dev')
  .description(getCmd('dev').desc)
  .action(() => {
    const { _ } = argv
    execCmd(_[0])
  })

program
  .command('daily')
  .description(getCmd('daily').desc)
  .action(() => {
    const { _ } = argv

    execCmd(_[0])
  })

program
  .command('dll')
  .description(getCmd('dll').desc)
  .action(() => {
    const { _ } = argv

    execCmd(_[0])
  })

program
  .command('build')
  .description(getCmd('build').desc)
  .action(() => {
    const { _ } = argv

    execCmd(_[0])
  })

program
  .command('babel')
  .description(getCmd('babel').desc)
  .action(() => {
    execa.shell(
      `rimraf ${process.cwd() + '/lib'} && cross-env BABEL_ENV=babel babel ${process.cwd() +
        '/src'} --out-dir ${process.cwd() + '/lib'} --copy-files --presets=${path.join(
        __dirname,
        '../babel-preset-abc.js'
      )} --no-babelrc`,
      { stdio: 'inherit', cwd: __dirname }
    )
  })

program
  .command('publish')
  .description(getCmd('publish').desc)
  .action(() => {
    execa.shell(`npm publish`, { stdio: 'inherit' })
  })

program.parse(process.argv)

if (program.generators) {
  printAvailableGenerators()

  process.exit(0)
}

const { _ } = argv
if (_.length === 0) {
  program.help()

  process.exit(0)
}
