/**
 * @fileOverview webpack基础配置
 * @author langford@126.com
 */

'use strict'

const webpack = require('webpack')
const autoprefixer = require('autoprefixer')
const _ = require('lodash')
const WebpackBar = require('webpackbar')
const GhostProgressWebpackPlugin = require('ghost-progress-webpack-plugin')
  .GhostProgressPlugin
const WebpackBuildNotifierPlugin = require('webpack-build-notifier')
const { VueLoaderPlugin } = require('vue-loader')

const utils = require('../utils')
const { abcConfig, paths } = utils
const { buildNotifier } = abcConfig
const cwd = process.cwd()

let plugins = [
  // new WebpackBar({
  //   profile: false
  // }),
  new GhostProgressWebpackPlugin(),
  new webpack.LoaderOptionsPlugin({
    options: {
      postcss: [
        autoprefixer({
          browsers: abcConfig.browsersList || utils.browsers
        })
      ],
      context: paths.appRootPath
    }
  }),
  new webpack.ContextReplacementPlugin(/moment[/\\]locale$/, /zh-cn/),
  new VueLoaderPlugin()
]

// 启用构建通知
if (buildNotifier.show) {
  plugins = [
    new WebpackBuildNotifierPlugin({
      title: buildNotifier.title,
      notifyOptions: {
        timeout: buildNotifier.timeout
      }
    }),
    ...plugins
  ]
}

module.exports = (config = {}) => {
  const { env } = config
  const envConfig = abcConfig.env[env]

  if (_.isEmpty(abcConfig.entry)) {
    if (env === 'development') {
      abcConfig.entry = { index: './demo/index.js' }
    }

    if (env === 'production' || env === 'daily') {
      abcConfig.entry = { index: './src/index.js' }
    }
  }

  return {
    entry: _.isEmpty(envConfig.entry) ? utils.getDevEntry(abcConfig.entry) : envConfig.entry,
    output: _.isEmpty(envConfig.output) ? abcConfig.output : envConfig.output,
    devtool: envConfig.devtool || abcConfig.devtool,
    module: {
      rules: utils.mergeRules(
        [
          {
            test: /\.jsx?$/,
            use: [
              {
                loader: 'babel',
                options: utils.getBabelOptions(config, envConfig)
              }
            ],
            exclude: /node_modules/
          },
          {
            test: /\.vue$/,
            use: [
              {
                loader: 'vue-loader'
              }
            ]
          },
          {
            test: /\.(jpe?g|png|gif)$/,
            use: [
              {
                loader: 'url',
                options: {
                  limit: 10 * 1024,
                  name: 'imgs/[name].[ext]'
                }
              }
            ]
          },
          {
            test: /\.svg$/,
            use: [
              {
                loader: 'svg-url',
                options: {
                  limit: 10 * 1024,
                  name: 'imgs/[name].[ext]',
                  noquotes: true
                }
              }
            ]
          },
          {
            test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
            use: [
              {
                loader: 'url',
                options: {
                  limit: 10 * 1024,
                  name: 'media/[name].[ext]'
                }
              }
            ]
          },
          {
            test: /\.(woff|svg|eot|ttf)\??.*$/,
            use: [
              {
                loader: 'url',
                options: {
                  limit: 1000,
                  name: 'fonts/[name].[ext]'
                }
              }
            ]
          },
          // {
          //   test: /\.html$/,
          //   use: [
          //     {
          //       loader: 'html',
          //       options: {
          //         minimize: true
          //       }
          //     }
          //   ]
          // },
          {
            test: /\.tsx?$/,
            exclude: /node_modules/,
            use: [
              {
                loader: 'babel',
                options: utils.getBabelOptions(config, envConfig)
              },
              {
                loader: 'awesome-typescript',
                options: {
                  transpileOnly: true
                }
              }
            ]
          }
        ],
        abcConfig.rules
      )
    },
    plugins: utils.mergePlugins(plugins, abcConfig.plugins),
    resolve: {
      modules: [
        paths.appNodeModulePath,
        paths.builderNodeModulePath,
        'node_modules'
      ],
      extensions: [
        '.js',
        '.vue',
        '.json',
        '.jsx',
        '.ts',
        '.tsx',
        '.less',
        '.scss',
        '.sass',
        '.styl'
      ],
      alias: _.merge(
        {
          demo: `${cwd}/demo`,
          src: `${cwd}/src`
        },
        abcConfig.alias
      )
    },
    resolveLoader: {
      modules: [
        paths.builderNodeModulePath,
        paths.appNodeModulePath,
        'node_modules'
      ],
      moduleExtensions: ['-loader']
    }
  }
}
