/**
 * @fileOverview webpack生产环境配置
 * @author langford@126.com
 */

"use strict"

const path = require("path")
const webpack = require("webpack")
const UglifyJsPlugin = require("uglifyjs-webpack-plugin")
const TerserPlugin = require('terser-webpack-plugin')
const HtmlWebpackPlugin = require("html-webpack-plugin")
const HtmlWebpackTagsPlugin = require('html-webpack-tags-plugin')
const CleanWebpackPlugin = require("clean-webpack-plugin")
const MiniCssExtractPlugin = require("mini-css-extract-plugin")
const BundleAnalyzerPlugin = require("webpack-bundle-analyzer")
  .BundleAnalyzerPlugin
const CopyWebpackPlugin = require("copy-webpack-plugin")
const OptimizeCSSAssetsPlugin = require("optimize-css-assets-webpack-plugin")
const md5File = require("md5-file")
const _ = require("lodash")
const config = require("./webpack.base.config")({
  dev: false,
  env: "production"
})
const utils = require("../utils")

const abcConfig = utils.abcConfig
const {
  dll,
  output,
  cleanBuild,
  analyzerReport,
  isExtractCss,
  cssModule,
  outputDir,
  staticPath
} = abcConfig
const prodConfig = abcConfig.env.production || {}

let rules = []

// 未启用 cssModule 并且 isExtractCss 设置为 true，生成独立的 css 文件
if (isExtractCss && !cssModule.enable) {
  rules = [
    {
      test: /\.css$/,
      use: [
        MiniCssExtractPlugin.loader,
        {
          loader: "css",
          options: {
            modules: false,
            minimize: true
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        }
      ]
    },
    {
      test: /\.less$/,
      use: [
        MiniCssExtractPlugin.loader,
        {
          loader: "css",
          options: {
            modules: false,
            minimize: true,
            importLoaders: 2
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        {
          loader: "less",
          options: {
            javascriptEnabled: true
          }
        }
      ]
    },
    {
      test: /\.s(c|a)ss$/,
      use: [
        MiniCssExtractPlugin.loader,
        {
          loader: "css",
          options: {
            modules: false,
            minimize: true
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        "sass"
      ]
    },
    {
      test: /\.styl(us)?$/,
      use: [
        MiniCssExtractPlugin.loader,
        {
          loader: "css",
          options: {
            modules: false,
            minimize: true
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        "stylus"
      ]
    }
  ]
} else {
  // 合并 production rules
  rules = [
    {
      test: /\.css$/,
      use: [
        "style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            minimize: true,
            localIdentName: abcConfig.cssModule.localIdentName
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        }
      ]
    },
    {
      test: /\.less$/,
      use: [
        "style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            minimize: true,
            localIdentName: abcConfig.cssModule.localIdentName,
            importLoaders: 2
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        {
          loader: "less",
          options: {
            javascriptEnabled: true
          }
        }
      ]
    },
    {
      test: /\.s(c|a)ss$/,
      use: [
        "style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            minimize: true,
            localIdentName: abcConfig.cssModule.localIdentName
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        "sass"
      ]
    },
    {
      test: /\.styl(us)?$/,
      use: [
        "vue-style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            minimize: true,
            localIdentName: abcConfig.cssModule.localIdentName
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        "stylus"
      ]
    }
  ]
}

rules = [rules, config.module.rules, prodConfig.rules || []]

config.module.rules = utils.merge(rules, "rule")

// 启用 dll 打包
if (dll.enable) {
  config.plugins = [
    new webpack.DllReferencePlugin({
      manifest: require(`${output.path}/vendor.manifest.json`)
    }),
    ...config.plugins
  ].concat(
    abcConfig.enableHtmlWebpackPlugin
      ? [
          new HtmlWebpackPlugin({
            template: prodConfig.htmlTemplateUrl || abcConfig.htmlTemplateUrl,
            inject: "body",
            ...abcConfig.htmlWebpackPluginOptions,
            ...prodConfig.htmlWebpackPluginOptions
          }),
          new HtmlWebpackTagsPlugin({
            tags: [`vendor.dll.js`],
            append: false,
            useHash: true
          })
        ]
      : []
  )
} else {
  config.plugins = [...config.plugins].concat(
    abcConfig.enableHtmlWebpackPlugin
      ? [
          new HtmlWebpackPlugin({
            template: prodConfig.htmlTemplateUrl || abcConfig.htmlTemplateUrl,
            inject: "body",
            ...abcConfig.htmlWebpackPluginOptions,
            ...prodConfig.htmlWebpackPluginOptions
          })
        ]
      : []
  )
}

// 清除打包目录，默认 vendor 相关文件不删除
if (cleanBuild) {
  config.plugins = [
    new CleanWebpackPlugin({
      cleanOnceBeforeBuildPatterns: [
        "**/*",
        "!vendor.dll.js",
        "!vendor.manifest.json"
      ]
    }),
    ...config.plugins
  ]
}

// 是否启用分析报告功能，方便进行打包后优化分析
if (analyzerReport) {
  config.plugins = [
    new BundleAnalyzerPlugin({
      analyzerPort: 1111,
      generateStatsFile: false
    }),
    ...config.plugins
  ]
}

// 启用了静态资源目录
if (staticPath) {
  config.plugins = [
    new CopyWebpackPlugin([
      { from: `${staticPath}`, to: `${output.path}`, toType: "dir" }
    ]),
    ...config.plugins
  ]
}

// 抽取出单独的 css 文件
if (isExtractCss && !cssModule.enable) {
  config.plugins = [
    new MiniCssExtractPlugin({
      ...{
        filename: "[name].[hash:5].css",
        chunkFilename: "[id].[hash:5].css"
      },
      ...abcConfig.minCssExtract,
      ...prodConfig.minCssExtract
    }),
    ...config.plugins
  ]
}

const plugins = [
  [
    new webpack.DefinePlugin({
      __DEV__: false,
      NODE_ENV: JSON.stringify("production")
    })
  ],
  new OptimizeCSSAssetsPlugin(),
  config.plugins || [],
  prodConfig.plugins || []
]

config.plugins = utils.merge(plugins, "plugin")
config.externals = _.merge(
  {},
  config.externals || {},
  abcConfig.externals || {},
  prodConfig.externals || {}
)
config.mode = "production"
config.optimization = {
  minimizer: [
    new TerserPlugin({
      parallel: true,
    })
  ]
}

module.exports = config
